import numpy as np


def edge(open: np.array, high: np.array, low: np.array, close: np.array) -> float:
    """
    Efficient Estimation of Bid-Ask Spreads from OHLC Prices

    Implements an efficient estimation procedure of the bid-ask spread from Open, High, Low, and Close
    prices as proposed in Ardia, Guidotti, Kroencke (2021): https://www.ssrn.com/abstract=3892335

    Prices must be sorted in ascending order of the timestamp.
    :param open: array-like vector of Open prices.
    :param high: array-like vector of High prices.
    :param low: array-like vector of Low prices.
    :param close: array-like vector of Close prices.

    :return: The spread estimate.
    """

    n = len(open)
    if len(high) != n or len(low) != n or len(close) != n:
        raise Exception("open, high, low, close must have the same length")

    o = np.log(np.asarray(open))
    h = np.log(np.asarray(high))
    l = np.log(np.asarray(low))
    c = np.log(np.asarray(close))
    m = (h + l) / 2.

    h1, l1, c1, m1 = h[:-1], l[:-1], c[:-1], m[:-1]
    o, h, l, c, m = o[1:], h[1:], l[1:], c[1:], m[1:]

    x1 = (m - o) * (o - m1) + (m - c1) * (c1 - m1)
    x2 = (m - o) * (o - c1) + (o - c1) * (c1 - m1)

    e1 = np.nanmean(x1)
    e2 = np.nanmean(x2)

    v1 = np.nanvar(x1)
    v2 = np.nanvar(x2)

    w1 = v2 / (v1 + v2)
    w2 = v1 / (v1 + v2)
    k = 4 * w1 * w2

    n1 = np.nanmean(o == h)
    n2 = np.nanmean(o == l)
    n3 = np.nanmean(c1 == h1)
    n4 = np.nanmean(c1 == l1)
    n5 = np.nanmean(np.logical_and(h == l, l == c1))

    s2 = -4 * (w1 * e1 + w2 * e2) / ((1 - k * (n1 + n2) / 2) + (1 - n5) * (1 - k * (n3 + n4) / 2))
    return float(max(0, s2) ** 0.5)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bidask']

package_data = \
{'': ['*']}

install_requires = \
['numpy']

setup_kwargs = {
    'name': 'bidask',
    'version': '0.1.0',
    'description': 'Efficient Estimation of Bid-Ask Spreads from Open, High, Low, and Close Prices',
    'long_description': '# Efficient Estimation of Bid-Ask Spreads from Open, High, Low, and Close Prices\n\nImplements an efficient estimation procedure of the bid-ask spread from Open, High, Low, and Close prices as proposed in Ardia, Guidotti, Kroencke (2021): https://www.ssrn.com/abstract=3892335\n\n## Installation\n\nInstall this package with:\n\n```bash\npip install bidask\n```\n\n## Usage\n\nImport the estimator\n\n```python\nfrom bidask import edge\n```\n\nEstimate the spread\n\n```python\nedge(open, high, low, close)\n```\n\n- `open`: array-like vector of Open prices.\n- `high`: array-like vector of High prices.\n- `low`: array-like vector of Low prices.\n- `close`: array-like vector of Close prices.\n\nPrices must be sorted in ascending order of the timestamp.\n\n## Cite as\n\n*Ardia, David and Guidotti, Emanuele and Kroencke, Tim Alexander, "Efficient Estimation of Bid-Ask Spreads from Open, High, Low, and Close Prices". Available at SSRN: https://ssrn.com/abstract=3892335*\n\nA BibTex  entry for LaTeX users is:\n\n```bibtex\n@unpublished{edge2021,\n    author = {Ardia, David and Guidotti, Emanuele and Kroencke, Tim},\n    title  = {Efficient Estimation of Bid-Ask Spreads from Open, High, Low, and Close Prices},\n    year   = {2021},\n    note   = {Available at SSRN}\n    url    = {https://ssrn.com/abstract=3892335}\n}\n```\n',
    'author': 'Emanuele Guidotti',
    'author_email': 'emanuele.guidotti@unine.ch',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/eguidotti/bidask',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
